<?php

namespace Drupal\synimport\Service;

use Drupal\Core\Config\FileStorage;

/**
 * Node import class.
 *
 * @internal
 *   For internal usage by the Commerce synimport module.
 */
class Node {
  /**
   * Logger variable.
   *
   * @var Logger
   */
  protected $log;

  /**
   * CreateEntity Service variable.
   *
   * @var CreateEntity
   */
  protected $createEntity;

  /**
   * Service constructor.
   *
   * @param CreateEntity $createEntity
   */
  public function __construct(CreateEntity $createEntity) {
    $this->createEntity = $createEntity;
  }

  /**
   * {@inheritdoc}
   */
  public function setLogger(Logger $log) {
    $this->log = $log;
  }
  
  /**
   * Search for created Node.
   */
  private function getNodeFromBase(string $unique_name) {
    $nodes = \Drupal::entityTypeManager()->getStorage('node')->loadByProperties(['field_unique_name' => $unique_name]);
    $node = array_shift($nodes);
    return $node ?? NULL;
  }

  /**
   * Search for nodes_array in $this->directory/nodes.
   */
  private function getNodeFromFile(string $unique_name) {
    $sourcenodes = new FileStorage($this->directory);
    foreach ($sourcenodes->listAll() as $file) {
      $node_array = $sourcenodes->read($file);
      if ($node_array['field_unique_name'] && $node_array['field_unique_name'] == $unique_name) {
        break;
      }
    }
    return $node_array;
  }

  /**
   * Import Nodes.
   */
  public function import(string $directory) {
    $this->directory = $directory;
    $sourcenodes = new FileStorage($directory);
    foreach ($sourcenodes->listAll() as $file) {
      $node_array = $sourcenodes->read($file);
      $this->updateCreateNode($node_array);
    }
  }

  /**
   * Update or Create Nodes.
   */
  private function updateCreateNode(array $node_array) {
    if ($node_array['field_unique_name']) {
      $created_node = $this->getNodeFromBase($node_array['field_unique_name']);
    } else {
      $created_node = FALSE;
    }
    if (!($created_node)) {
      $this->createNode($node_array, 'full_create');
    }
    elseif (!empty($node_array['field_related_projects'])) {
      $created_node_id = $created_node->id();
      $this->setRelated($created_node_id, $node_array['field_related_projects']);
    }
    elseif (!empty($node_array['field_related_portfolios'])) {
      $created_node_id = $created_node->id();
      $this->setRelated($created_node_id, $node_array['field_related_portfolios']);
    }
  }

  /**
   * Create Node.
   */
  private function createNode(array $node_array, string $statement = 'full_create') {
    if ($statement == 'full_create') {
      $this->createEntity->setLogger($this->log);
      $created_node_id = $this->createEntity->create($node_array, 'node')->id();
      if (array_key_exists('is_front', $node_array) && $node_array['is_front'] == TRUE) {
        $config = \Drupal::service('config.factory')->getEditable('system.site');
        $config->set('page.front', "/node/$created_node_id");
        $config->save();
      }
      if (!empty($node_array['field_related_projects'])) {
        $this->setRelated($created_node_id, $node_array['field_related_projects'], 'field_related_projects');
      }
      if (!empty($node_array['field_related_portfolios'])) {
        $this->setRelated($created_node_id, $node_array['field_related_portfolios'], 'field_related_portfolios');
      }
    }
    elseif ($statement == 'only_as_related') {
      $created_node = $this->getNodeFromBase($node_array['field_unique_name']);
      if ($created_node) {
        return $created_node->id();
      }
      $this->createEntity->setLogger($this->log);
      $created_node_id = $this->createEntity->create($node_array, 'node')->id();
    }
    return $created_node_id;
  }

  /**
   * Create Related Nodes.
   */
  private function setRelated(string $created_node_id, array $related_nodes, string $field_name) {
    $storage = \Drupal::entityTypeManager()->getStorage('node');
    $nodes_ids = [];
    foreach ($related_nodes as $unique_name) {
      $node_array = $this->getNodeFromFile($unique_name);
      if (!empty($node_array)) {
        $nodes_ids[] = $this->createNode($node_array, 'only_as_related');
      }
    }
    /** @var \Drupal\node\Entity\nodeInterface $created_node */
    $created_node = $storage->load($created_node_id);
    if ($field_name == 'field_related_projects') {
       $created_node->set($field_name, $nodes_ids);
    } elseif ($field_name == 'field_related_portfolios') {
       $created_node->set($field_name, $nodes_ids);
    }
    $created_node->save();
  }

}
